% The BSVM package includes a main program (BSVM_main.m) and three functions
%(Simulation_T1D.m, SVM.m, Permutation.m) as follows.
% 'Simulation_TID.m' is a function for generating Type 1 diabetes data.
% 'SVM.m' is a function for support vector classification.
% 'Permutation.m' is a function for permutation procedure.

clear; 
% Parameters setting of simulation data   
Num_ind=480; % # case = # control = 480
Replication=100; % # of replications
 
% Random seed  
A=clock;        
AA=ceil(A(6));       
SIMU=AA;    
while SIMU<AA+Replication; % # of replications     
      POISS=[];
      Poiss=[];
      %% Generate case-control data
      [Simu_asp]=Simulation_DIT(Num_ind); 
     
      Num_var=length(Simu_asp(1,:))-2; % Find # of variants
      ASP_CHILD=Simu_asp; 
      Child_case=[]; 
      Child_control=[]; 
      Child_case_code=[]; 
      Child_control_code=[];
      % Separate cases and controls
      Child_case=ASP_CHILD(find(ASP_CHILD(:,end)==1),:);
      Child_control=ASP_CHILD(find(ASP_CHILD(:,end)==0),:);
 
      % Detect risk variant & find important variant
       for i=1:length(Child_case(1,:))-2
          a(i)=length(find(Child_case(:,i)==1))+2*length(find(Child_case(:,i)==2));
          b(i)=length(find(Child_control(:,i)==1))+2*length(find(Child_control(:,i)==2));
          % Check risk or protective variant
          if binocdf(a(i),a(i)+b(i),0.5) > 1-binocdf(a(i),a(i)+b(i),0.5); % Risk
             POISS(1,i)=[poisspdf(1,(a(i)+1)/(2*Num_ind+2))]/[poisspdf(1,(b(i)+1)/(2*Num_ind+2))];
             POISS(2,i)=2;% Mark this variant as risk variant
          else binocdf(a(i),a(i)+b(i),0.5) <= 1-binocdf(a(i),a(i)+b(i),0.5)  ; 
             POISS(1,i)=[poisspdf(1,(b(i)+1)/(2*Num_ind+2))]/[poisspdf(1,(a(i)+1)/(2*Num_ind+2))];
             POISS(2,i)=1;
          end  
       end 
      
      POISS(3,1:Num_var)=[1:Num_var];
      Poiss=sortrows(POISS',1);    
      Poiss(find(Poiss(:,1)==0),:)=[];
      Poiss1=Poiss(find(Poiss(:,2)==1),:);
      Poiss2=Poiss(find(Poiss(:,2)==2),:); 
      Poiss2(find(Poiss2(:,1)==0),:)=[];
      % Compute W*X 
      for i=1:Num_var
          ASP_CHILD(find(ASP_CHILD(:,i)==1),i)=POISS(1,i); %case
          ASP_CHILD(find(ASP_CHILD(:,i)==2),i)=2*POISS(1,i);
      end   
       
      %%%%%%% BSVM learning procedure %%%%%%%%%%
      acc=[];    
      SVMTrainX=[];  
      ind_na=length(Poiss1(:,1)); 
      ind_po=length(Poiss2(:,1));
      ind=length(Poiss(:,1));
      IND=1;
     for i=1:Num_ind*2
       SVMTrainX(i,2)=sum(ASP_CHILD(i,Poiss2(end-ind_po+1:end,3)'));
       SVMTrainX(i,1)=sum(ASP_CHILD(i,Poiss1(end-ind_na+1:end,3)'));
     end 
       SVMTrainY=ASP_CHILD(:,Num_var+2);
       jj=1/max(Poiss(length(Poiss(:,1))-ind+1:end,2));
       % All variants's learning
       [accuracy]=SVM(SVMTrainY, SVMTrainX,jj);
       acc( ind+1)=accuracy(3); 
       ACC=0;
       Diff_acc=acc( ind+IND)-ACC;  
       while  Diff_acc>=0; % If diff. of R^2 of backward SVM is large >0 then delete variants
           for j=1:ind 
               if Poiss(j,2)==2 % Put all variants into risk confounder
                  for i=1:2*Num_ind
                      SVMTrainX(i,2)=sum(ASP_CHILD(i,setdiff(Poiss2(:,3)', Poiss(j,3)) ));
                      SVMTrainX(i,1)=sum(ASP_CHILD(i,Poiss1(:,3)')); 
                  end
               else % Put all variants into protective confounder
                  for i=1:2*Num_ind
                      SVMTrainX(i,1)=sum(ASP_CHILD(i,setdiff(Poiss1(:,3)', Poiss(j,3)) ));
                      SVMTrainX(i,2)=sum(ASP_CHILD(i,Poiss2(:,3)'));   
                  end
               end 
               jj=0.5; % Suggestion for SVM parameter setting
               % Compute new R^2
               [accuracy]=SVM(SVMTrainY, SVMTrainX,jj);
               acc(j)=accuracy(3);   
           end             
           % If diff. of R^2 of backward SVM is large >0 then delete variants
           Diff_acc=max(acc(1:ind))-acc(ind+IND); 
           if Diff_acc>=0      
               if Poiss(min(find(acc==max(acc(1:ind)))),2)==1
                  Poiss1(find(Poiss1(:,3)==Poiss(min(find(acc==max(acc(1:ind)))),3)),:)=[];
                  Poiss(find(Poiss(:,3)==Poiss(min(find(acc==max(acc(1:ind)))),3)),:)=[];
                  acc(ind+IND)=max(acc(1:ind));
                  acc(1:ind)=0;
                  ind=ind-1;
                  IND=IND+1;     
               else
                  Poiss2(find(Poiss2(:,3)==Poiss(min(find(acc==max(acc(1:ind)))),3)),:)=[];
                  Poiss(find(Poiss(:,3)==Poiss(min(find(acc==max(acc(1:ind)))),3)),:)=[];
                  acc(ind+IND)=max(acc(1:ind));
                  acc(1:ind)=0;
                  ind=ind-1;
                  IND=IND+1;
              end    
           end
       end  
       %%%%%%% BSVM learning procedure is finished %%%%%%%%%%
       % SVM learning after variant selection is finished 
       for i=1:Num_ind*2
           SVMTrainX(i,2)=sum(ASP_CHILD(i,Poiss2(:,3)')); % Risk confounder
           SVMTrainX(i,1)=sum(ASP_CHILD(i,Poiss1(:,3)')); % Protective confounder
       end     
      [accuracy]=SVM(SVMTrainY, SVMTrainX,jj);  % Use SVM to compute R^2
      Max_acc=accuracy(3);  % The max R^2 after BSVM procedure
       %%%%%% Permutation fot p-value %%%%%%
      C11=0;                                 
      [pvalue_svm]=permutation(C11,Max_acc, Simu_asp,Num_ind,Num_var);
      P_value_svm(SIMU-AA+1)=pvalue_svm;  
      
      % Detect non-redundant    
      aaa=length(Poiss(:,1));
      PPP(1:aaa,SIMU-AA+1)=Poiss(:,3);
      SIMU=SIMU+1;                       
end 
Power_SVM=length(find(P_value_svm<0.05))/Replication % Power 
Mean_pvalue_SVM=mean(P_value_svm) % Average p-value